# File trwnavs.py
# Part of the trgensit package.
#
# * Functionality
#
# Generates the auxiliary navigation file trWNav.xml
# The file body.htm is expected in the source directory (=working directory)
#
# There must be a top level body.html file which is marked as the "home" one.
# The "bread crumb" is the path from that home directory down to the working directory.
#
#
# Copyright 2019-2020 Dr. Thomas Redelberger, redetho@gmx.de
# This code is open source according to GPL2.
# See file COPYING.TXT
#
# * Pre-requisites
#
# - Nothing beyond python
#
# * My suffix naming convention for variables in this file
#
# DirOb:        pathlib directory objects
# Elm:          ElementTree XML element objects
# Dir:          Tuple with element [0] being a pathlib directory object;
#               tuple elements [1] - [6] are of various types
# Dirs:         Tuple of Dir tuples
#
#
# $Id: trwnavs.py 1.1 2020/03/20 09:11:43 redetho Exp redetho $


import pathlib
import xml.etree.ElementTree as ET

import trwlib




def findBodyFile(thisDirOb):
    "This function finds the source body.htm file and collects its meta data"

    fileOb = thisDirOb.joinpath(trwlib.srcNamStr)
    try:
        treeOb = ET.parse(str(fileOb))  # open accepts pathlib object only >= 3.6
    except Exception as e:
        # e.errno is avail. for file related erors, but not for parse related errors
        print('{}: Error: File >>{}<< cannot be opened or parsed'.format(__file__, thisDirOb.joinpath(trwlib.srcNamStr)) )
        quit(trwlib.myErrNo)   # no point to continue

    
    titleElm = treeOb.find("head/title")
    titleStr = titleElm.text if titleElm is not None else ''

    tabMeta = treeOb.find("head/meta[@name='htTab']")
    tabStr = tabMeta.attrib['content'] if tabMeta is not None else ''

    cntMeta = treeOb.find("head/meta[@http-equiv='refresh']")
    fwdURIStr = cntMeta.attrib['content'].split('=',1)[1] if cntMeta is not None else ''

    homMeta = treeOb.find("head/meta[@name='htHom']")
    isHomeBl =  (homMeta is not None)

    ignoreMeta = treeOb.find("head/meta[@name='htIgnore']")
    ignoreBl = (ignoreMeta.attrib['content'] in ('y', 'Y', 'yes', 'Yes')) if ignoreMeta is not None else False
        
    orderMeta = treeOb.find("head/meta[@name='htOrder']")
    orderStr = orderMeta.attrib['content'] if orderMeta is not None else tabStr
    #          0         1        2         3         4         5         6
    return thisDirOb, titleStr, tabStr, fwdURIStr, isHomeBl, ignoreBl, orderStr



def getOrderedChildDirs(thisDirOb):
    "This function returns an /ordered/ tuple of child directory tuples"

    unorderedDirs = []          # will hold all children of this parent, in undefined order
    for childDirOb in thisDirOb.iterdir():
        if not childDirOb.is_dir(): continue
        
        if (childDirOb.parts[-1] == 'RCS'): continue    # ignore special directories

        tabDir = findBodyFile(childDirOb)
        # ignore marked directories or those where body.htm does not have a TabName
        if tabDir[5] or tabDir[2] == '': continue

        unorderedDirs.append(tabDir)    # store this child
        
    # bring the children in the wanted order
    return sorted(unorderedDirs, key=lambda myDir: myDir[6])   



def getTgtPath(tgtDirOb, thisDirOb):
    "This function calculates the shortest relative path from this to tgt"

    # The path to the target can either be shorter (if tgt is "above"), same (if "sibling"
    # or longer (if child)
    relLen = len(thisDirOb.parts) - len(tgtDirOb.parts)
    if relLen < 0:      # tgt is "below" i.e. a child
        return tgtDirOb.parts[-1]
    else:
        if relLen == 0: # is a sibling or the dir itself
            if tgtDirOb.parts[-1] == thisDirOb.parts[-1]:       # dir itself
                return '.'
            else:
                return '../' + tgtDirOb.parts[-1]               # a sibling
        else:   # tgt is "above"
            commIdx = len(tgtDirOb.parts) - 1
            #print('{} {} {}'.format(relLen, thisDirOb.parts[commIdx], tgtDirOb.parts[commIdx]) )
            if thisDirOb.parts[commIdx] == tgtDirOb.parts[commIdx]:
                return '..' + '/..' * (relLen - 1)      # optimise
            else:
                return '../' * (relLen + 1) + tgtDirOb.parts[-1]



#
# Explanation of arguments:
#
# homeDir: tuple of the "home" directory, where the top most body.htm file lives
# workDir: tuple of the directory for which the trWNav.xml file shall be generated
# tgtDir:  tuple of the roving directory on the bread crumb plus the siblings,
#          which are navigation /targets/ for workDir

def putNode(homeDir, workDir, parentElm, tgtDir):
    "This function puts the info for all /related/ nodes in the XML tree"
    "Related nodes are all nodes, whose /parents/ are on the bread crumb"

    # check whether the tgtDir is on the bread crumb of workDir
    inTreeBl =  workDir[0].parts[:len(tgtDir[0].parts)] == tgtDir[0].parts
    inTree = 'y' if inTreeBl else 'n'

    # find the children of this tgtDir
    orderedDirs = getOrderedChildDirs(tgtDir[0])
    hasChilds = 'n' if len(orderedDirs) == 0 else 'y'

    # mark node if this is the current node
    isCurrent = 'y' if tgtDir[0] == workDir[0] else 'n'

    # establish level of this tgt dir versus the home dir
    levelNo = 1 + len(tgtDir[0].relative_to(homeDir[0]).parts)
    if levelNo == 1:
        upDirStr = "."
    else:
        if levelNo == 2:
            upDirStr = ".."
        else:
            upDirStr = ".." + "/.." * (levelNo - 2)
  
    childElm = ET.SubElement(parentElm, 'htNode', {'hasChilds':hasChilds, 'levelNo':'{}'.format(levelNo), 'inTree':inTree, 'isCurrent':isCurrent})
    ET.SubElement(childElm, 'Title').text = tgtDir[1]
    ET.SubElement(childElm, 'TabName').text = tgtDir[2]
    ET.SubElement(childElm, 'Dir').text = tgtDir[0].parts[-1]
    ET.SubElement(childElm, 'TgtURI').text = getTgtPath(tgtDir[0], workDir[0])
    ET.SubElement(childElm, 'DownDir').text = tgtDir[0].relative_to(homeDir[0]).as_posix()
    ET.SubElement(childElm, 'UpDir').text = upDirStr

    # if the /parent/ (path object tgtDir[0]) of these child directories is on the bread crumb,
    # then include its children recursively

    if inTreeBl:
        for childDir in orderedDirs:
            putNode(homeDir, workDir, childElm, childDir)



def putWork(homeDir, workDir):
    "This function puts the meta info about the 'work' node in the XML tree"

    levelNo = 1 + len(workDir[0].relative_to(homeDir[0]).parts)
    if levelNo == 1:
        upDirStr = "."
    else:
        if levelNo == 2:
            upDirStr = ".."
        else:
            upDirStr = ".." + "/.." * (levelNo - 2)

    docElm = ET.Element('htWeb2')
    ET.SubElement(docElm, 'htHomeDir').text = homeDir[0].parts[-1]
    ET.SubElement(docElm, 'htTitle').text = workDir[1]
    ET.SubElement(docElm, 'htTabName').text = workDir[2]
    ET.SubElement(docElm, 'htDir').text = workDir[0].parts[-1]
    ET.SubElement(docElm, 'htDownDir').text = workDir[0].relative_to(homeDir[0]).as_posix()
    ET.SubElement(docElm, 'htUpDir').text = upDirStr
    ET.SubElement(docElm, 'htLevelNo').text = '{}'.format(levelNo)
    ET.SubElement(docElm, 'htIgnore').text = 'y' if workDir[5] else 'n'
    ET.SubElement(docElm, 'htOrder').text = workDir[6]

    putNode(homeDir, workDir, docElm, homeDir)  # last arg = first tgt is the home directory
    
    #ET.dump(docElm)
    # write the result navigation file as XML
    try:
        ET.ElementTree(docElm).write(str(workDir[0].joinpath(trwlib.navNamStr)), encoding="UTF-8", xml_declaration=True)
    except Exception as e:
        print('{}: Error {}: File >>{}<< cannot be written'.format(__file__, e.errno, workDir[0].joinpath(trwlib.navNamStr)) )
        quit(trwlib.myErrNo)   # no point to continue


    

def generateNavFile(workDirOb):
    "This function generates the trWNav.xml file in the directory workDirOb"

    # find the home directory. Start with the work dir. and go up the directory tree
    workDir = findBodyFile(workDirOb)
    homeDirOb = workDirOb
    homeDir = workDir

    while not homeDir[4]:       # if it has isHomeBl set, I found it
        homeDirOb = homeDirOb.parent
        homeDir = findBodyFile(homeDirOb)

    putWork(homeDir, workDir)


                
        

############### main #################
# When used as a script, pass current working directory as argument

if __name__ == "__main__":
    # execute only if run as a script
    generateNavFile(pathlib.Path.cwd())


######################## end of trwnavs.py ###########################
